﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using System.Xml.Linq;
using System.Diagnostics;
using System.Text.RegularExpressions;
using Wiedemann;
using mslgen.Properties;

namespace mslgen
{
    public partial class FormMslGen : Form
    {
        // Rozmer nahledu.
        const int thumbnailSize = 58;

        // Fronta pro vytvoreni nahledu pouzita tridou QueuedBackgroundWorker.
        private Queue<QueueItem<BackgroundWorkerParameters>> thumbnailQueue = new Queue<QueueItem<BackgroundWorkerParameters>>();

        // Nastaveni aplikace.
        private Settings AppSettings;

        public FormMslGen()
        {
            InitializeComponent();

            maskedTextBoxJpgQuality.Mask = "000";

            // Nacteni konfigurace.
            AppSettings = Settings.Default;

            // Obnoveni posledniho stavu.
            checkBoxRecurseDirectory.Checked = AppSettings.RecurseDirectory;
            checkBoxTypeBmp.Checked = AppSettings.SelectedTypeBmp;
            checkBoxTypeGif.Checked = AppSettings.SelectedTypeGif;
            checkBoxTypeJpg.Checked = AppSettings.SelectedTypeJpg;
            checkBoxTypePng.Checked = AppSettings.SelectedTypePng;
            checkBoxTypeTga.Checked = AppSettings.SelectedTypeTga;
            checkBoxTypeTiff.Checked = AppSettings.SelectedTypeTiff;
            
            if (AppSettings.SaveAsIndex >= 0 && AppSettings.SaveAsIndex < comboBoxTargetFormat.Items.Count)
                comboBoxTargetFormat.SelectedIndex = AppSettings.SaveAsIndex;
            else
                comboBoxTargetFormat.SelectedIndex = 0;
            
            maskedTextBoxJpgQuality.Text = AppSettings.JpgQuality.ToString();
            
            if (File.Exists(AppSettings.ConjurePath))
                textBoxConjurePath.Text = AppSettings.ConjurePath;

            maskedTextBoxDestWidth.Text = AppSettings.DestWidth.ToString();
            maskedTextBoxDestHeight.Text = AppSettings.DestHeight.ToString();
            checkBoxForceRgb.Checked = AppSettings.ForceRgb;

            textBoxSaveImageMask.Text = AppSettings.SaveImageMask;

            switch (AppSettings.ResizeMode)
            {
                case 1:
                    radioButtonPixel.Checked = true;
                    break;
                case 2:
                    radioButtonPercent.Checked = true;
                    break;
            }

            // Vychozi nastaveni prvku ve status baru.
            toolStripStatusLabel.Text = "";
            toolStripProgressBar.Visible = false;
            toolStripDropDownButton.Visible = false;

            dataGridViewFiles.Sort(dataGridViewFiles.Columns["FileName"], ListSortDirection.Ascending);
        }

        List<string> GetSelectedExtensions()
        {
            List<string> selectedExtensions = new List<string>();

            if (checkBoxTypeBmp.Checked)
                selectedExtensions.Add("*.bmp");

            if (checkBoxTypeGif.Checked)
                selectedExtensions.Add("*.gif");

            if (checkBoxTypeJpg.Checked)
                selectedExtensions.Add("*.jpg");

            if (checkBoxTypePng.Checked)
                selectedExtensions.Add("*.png");

            if (checkBoxTypeTga.Checked)
                selectedExtensions.Add("*.tga");

            if (checkBoxTypeTiff.Checked)
                selectedExtensions.Add("*.tif");

            return selectedExtensions;
        }

        private void buttonBrowseDir_Click(object sender, EventArgs e)
        {
            // Vyber adresare s obrazky.
            using (FolderBrowserDialog folderDialog = new FolderBrowserDialog())
            {
                // Nabidnuti posledniho vybraneho adresare, pokud existuje.
                if (AppSettings.LastSelectedDirectory.Length > 0
                    && Directory.Exists(AppSettings.LastSelectedDirectory))
                    folderDialog.SelectedPath = AppSettings.LastSelectedDirectory;

                if (folderDialog.ShowDialog() == DialogResult.OK)
                {
                    // Ulozeni vybraneho adresare.
                    AppSettings.LastSelectedDirectory = folderDialog.SelectedPath;

                    // Nacteni souboru z vybraneho adresare.
                    ImportDirectory(folderDialog.SelectedPath);
                }
            }
        }

        private void ImportDirectory(string sourceDirectory)
        {
            // Pro jistotu, pokud by bezelo generovani nahledu.
            thumbnailQueue.Clear();

            // Vymazani vsech zaznamu v dataGridViewFiles.
            dataGridViewFiles.Rows.Clear();

            // Seznam pripon obrazku.
            List<string> selectedExtensions = GetSelectedExtensions();

            // Seznam nalezenych souboru.
            List<string> foundFiles = new List<string>();

            // Vyhledani obrazku.
            try
            {
                foreach (string extension in selectedExtensions)
                {
                    foundFiles.AddRange(Directory.GetFiles(sourceDirectory, extension,
                        checkBoxRecurseDirectory.Checked ? SearchOption.AllDirectories : SearchOption.TopDirectoryOnly));
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (foundFiles.Count > 0)
            {
                // Serazeni podle abecedy.
                DataGridViewColumn sortColumn = dataGridViewFiles.SortedColumn;
                if (sortColumn != null)
                {
                    foundFiles.Sort();

                    if (dataGridViewFiles.SortOrder == SortOrder.Descending)
                        foundFiles.Reverse();
                }

                // Inicializace tatusbaru.
                toolStripStatusLabel.Text = "Příprava náhledů ...";
                toolStripProgressBar.Value = 0;
                toolStripProgressBar.Visible = true;
                toolStripDropDownButton.Visible = true;
            }

            // Pridani nalezenych souboru do dataGridViewFiles.
            try
            {
                dataGridViewFiles.Rows.Add(foundFiles.Count);
                for (int row = 0; row < foundFiles.Count; row++)
                {
                    dataGridViewFiles.Rows[row].Tag = row;

                    // Sloupec Export.
                    dataGridViewFiles.Rows[row].Cells["Export"].Value = 1;

                    // Sloupec FileName.
                    dataGridViewFiles.Rows[row].Cells["FileName"].Value = foundFiles[row];

                    // Pridani obrazku do frony k vytvoreni nahledu.
                    GetThumbnailInBackground(row, foundFiles[row]);
                }
            }
            catch (ArgumentOutOfRangeException)
            {
                // Nebyly nalezeny zadne soubory. Tuto vyjimku
                // vyvola metoda Add.
                toolStripStatusLabel.Text = "";
                toolStripProgressBar.Visible = false;
                toolStripDropDownButton.Visible = false;

                MessageBox.Show("Nebyly nalezeny žádné soubory.", Text, MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }

            // Ulozeni vybraneho adresare
            textBoxDirectory.Text = sourceDirectory;
        }

        private void GetThumbnailInBackground(int rowId, string fileName)
        {
            BackgroundWorkerParameters bwParams = new BackgroundWorkerParameters();
            bwParams.rowId = rowId;
            bwParams.fileName = fileName;

            // Zarazeni pozadavku na vytvoreni noveho nahledu do fronty.
            QueuedBackgroundWorker.QueueWorkItem(
                thumbnailQueue, bwParams,
                args =>
                {
                    try
                    {
                        // Otevreni zdrojoveho obrazku.
                        using (Image image = Image.FromFile(args.Argument.fileName))
                        {
                            // Vypocet rozmeru nahledu.
                            double ratio;
                            int newWidth, newHeight;

                            ratio = (double)image.Width / (double)image.Height;
                            if (ratio > 1.0)
                            {
                                newWidth = thumbnailSize;
                                newHeight = (int)(Convert.ToDouble(thumbnailSize) / ratio);
                            }
                            else
                            {
                                newWidth = (int)(Convert.ToDouble(thumbnailSize) * ratio);
                                newHeight = thumbnailSize;
                            }

                            // Vytvoreni nove bitmapy s rozmery nahledu.
                            Bitmap imageThumbnail = new Bitmap(newWidth, newHeight);

                            // Zmenseni zdrojoveho obrazku na rozmery nahledu.
                            using (Graphics graphics = Graphics.FromImage(imageThumbnail))
                            {
                                // Nastaveni kvality nahledu.
                                graphics.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;
                                graphics.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
                                graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
                                
                                // Vykresleni nahledu.
                                graphics.DrawImage(image, 0, 0, imageThumbnail.Width, imageThumbnail.Height);
                            }

                            // Ulozeni nahledu z BackgroundWorkeru pro
                            // zpracovani ve vlaknu s DataGridView.
                            args.Argument.imageThumbnail = imageThumbnail;
                        }
                    }
                    catch
                    {
                        // V pripade, ze behem otevirani nebo zmensovani
                        // obrazku nastane chyba, nastavime nahled
                        // na null a ve vlakne s DataGridView tento
                        // obrazek budeme ignorovat.
                        args.Argument.imageThumbnail = null;
                    }

                    return args.Argument;
                },
                args =>
                {
                    if (args.Result.imageThumbnail != null)
                    {
                        // Nalezeni pozice radku v DataGridView. Nemusi se shodovat
                        // s rowID, pokud uzivatel data seradil.
                        try
                        {
                            var rowPosition = (from dgRow in dataGridViewFiles.Rows.Cast<DataGridViewRow>()
                                               where Convert.ToInt32(dgRow.Tag) == args.Result.rowId
                                               select dgRow.Index).First();

                            dataGridViewFiles.Rows[rowPosition].Cells["Thumbnail"].Value = args.Result.imageThumbnail;

                            // Aktualizace ProgressBaru.
                            int percentDone = ++args.Result.rowId * 100 / dataGridViewFiles.Rows.Count;
                            if (percentDone != toolStripProgressBar.Value)
                            {
                                toolStripProgressBar.Value = percentDone;
                            }
                        }
                        catch (InvalidOperationException)
                        {
                        }
                    }
                    else
                    {
                        // V pripade, ze generovani nahledu selhalo,
                        // pouze zvysime rowID, aby mohla uspet nasledujici kontrola.
                        ++args.Result.rowId;
                    }

                    // Vsechny nahledy jsou vytvoreny ... skryti progressbaru.
                    if (args.Result.rowId == dataGridViewFiles.Rows.Count)
                    {
                        toolStripStatusLabel.Text = "";
                        toolStripProgressBar.Visible = false;
                        toolStripDropDownButton.Visible = false;
                    }
                });
        }

        private void toolStripDropDownButton_Click(object sender, EventArgs e)
        {
            // Uzivatel zrusil generovani nahledu.
            thumbnailQueue.Clear();

            toolStripStatusLabel.Text = "";
            toolStripProgressBar.Visible = false;
            toolStripDropDownButton.Visible = false;
        }

        private void buttonBrowseConjure_Click(object sender, EventArgs e)
        {
            // Vyhledani programu conjure.
            using (OpenFileDialog openDlg = new OpenFileDialog())
            {
                openDlg.Filter = "conjure.exe|conjure.exe";
                openDlg.FilterIndex = 1;
                openDlg.RestoreDirectory = true;

                if (openDlg.ShowDialog() == DialogResult.OK)
                {
                    AppSettings.ConjurePath = textBoxConjurePath.Text = openDlg.FileName;
                }
            }
        }

        private void FormMslGen_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Ulozeni nastaveni.
            int intVal;

            AppSettings.RecurseDirectory = checkBoxRecurseDirectory.Checked;
            AppSettings.SelectedTypeBmp = checkBoxTypeBmp.Checked;
            AppSettings.SelectedTypeGif = checkBoxTypeGif.Checked;
            AppSettings.SelectedTypeJpg = checkBoxTypeJpg.Checked;
            AppSettings.SelectedTypePng = checkBoxTypePng.Checked;
            AppSettings.SelectedTypeTga = checkBoxTypeTga.Checked;
            AppSettings.SelectedTypeTiff = checkBoxTypeTiff.Checked;

            AppSettings.SaveAsIndex = comboBoxTargetFormat.SelectedIndex;
            
            if (!Int32.TryParse(maskedTextBoxJpgQuality.Text, out intVal))
                intVal = 95;
            AppSettings.JpgQuality = intVal;
            
            AppSettings.ConjurePath = textBoxConjurePath.Text;

            if (!Int32.TryParse(maskedTextBoxDestWidth.Text, out intVal))
                intVal = 0;
            AppSettings.DestWidth = intVal;

            if (!Int32.TryParse(maskedTextBoxDestHeight.Text, out intVal))
                intVal = 0;
            AppSettings.DestHeight = intVal;

            AppSettings.SaveImageMask = textBoxSaveImageMask.Text.Trim();

            if (radioButtonPixel.Checked)
                AppSettings.ResizeMode = 1;
            else if (radioButtonPercent.Checked)
                AppSettings.ResizeMode = 2;

            AppSettings.Save();
        }

        private void buttonMarkAll_Click(object sender, EventArgs e)
        {
            // Nastaveni priznaku ve sloupci Export u vsech zaznamu.
            for (int row = 0; row < dataGridViewFiles.Rows.Count; row++)
            {
                dataGridViewFiles.Rows[row].Cells["Export"].Value = 1;
            }
        }

        private void buttonUnmarkAll_Click(object sender, EventArgs e)
        {
            // Zruseni priznaku ve sloupci Export u vsech zaznamu.
            for (int row = 0; row < dataGridViewFiles.Rows.Count; row++)
            {
                dataGridViewFiles.Rows[row].Cells["Export"].Value = 0;
            }
        }

        private void buttonStartConjure_Click(object sender, EventArgs e)
        {
            // Kontrola, jestli je vubec co zmensovat.
            int selectedFilesCount = 0;
            foreach (DataGridViewRow row in dataGridViewFiles.Rows)
            {
                try
                {
                    if (Convert.ToInt32(row.Cells["Export"].Value) == 1)
                        selectedFilesCount++;
                }
                catch (InvalidCastException)
                {
                }
            }

            // Nejsou nacteny zadne obrazky nebo neni vybran zadny pro export.
            if (selectedFilesCount == 0)
            {
                MessageBox.Show("Nejsou načteny obrázky nebo není žádný vybrán pro export.", Text, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            // Kontrola, jestli je jako 1. znak zpetne lomitko. Pokud ano, je odstraneno.
            if (textBoxSaveImageMask.Text.Trim().StartsWith("\\"))
                textBoxSaveImageMask.Text = textBoxSaveImageMask.Text.Trim().TrimStart('\\').Trim();

            // Kontrola, jestli je maska spravne.
            Regex regFileMask = new Regex(".*\\[:NAME:\\][^\\\\]*\\[:EXT:\\]$");
            if (!regFileMask.IsMatch(textBoxSaveImageMask.Text.Trim()))
            {
                MessageBox.Show("Maska pro uložení souboru není správně. Musí obsahovat proměnné [:NAME:] a [:EXT:]. Zároveň mezi nimi nesmí být zpětné lomítko a proměnná [:EXT:] musí být na konci textu.",
                    Text, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            // Kontrola, jestli se budou prepisovat puvodni obrazky.
            if (textBoxSaveImageMask.Text.Trim() == "[:NAME:][:EXT:]")
            {
                if (MessageBox.Show("Maska pro uložení souborů je nastavena pro přepsání původních souborů. Přesto pokračovat?",
                    Text, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.No)
                    return;
            }

            // Kontrola, jestli je zadan alespon 1 rozmer, pokud nemenime format.
            int destWidth, destHeight;

            if (!Int32.TryParse(maskedTextBoxDestWidth.Text, out destWidth))
                destWidth = 0;
            if (!Int32.TryParse(maskedTextBoxDestHeight.Text, out destHeight))
                destHeight = 0;

            if (destWidth == 0 && destHeight == 0 && comboBoxTargetFormat.SelectedIndex == 0)
            {
                MessageBox.Show("Musí být zadán alespoň 1 rozměr.", Text, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            // Kontrola, jestli je zadana cesta k programu conjure.
            if (textBoxConjurePath.Text.Trim().Length == 0
                || !File.Exists(textBoxConjurePath.Text.Trim()))
            {
                MessageBox.Show("Zadejte cestu k programu conjure.", Text, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            // Vysledne rozmery obrazku.
            StringBuilder destinationGeometry = new StringBuilder();
            if (destWidth > 0 || destHeight > 0)
            {
                if (destWidth > 0)
                    destinationGeometry.Append(destWidth.ToString());
                
                if (destHeight > 0)
                {
                    destinationGeometry.Append("x");
                    destinationGeometry.Append(destHeight.ToString());
                }

                if (radioButtonPercent.Checked)
                    destinationGeometry.Append("%");
                else if (destWidth > 0 && destHeight > 0)
                    destinationGeometry.Append(">");
            }

            // Pokud je v masce souboru zpetne lomitko, musime
            // predem vytvorit podadresare, kam budou obrazky ulozeny.
            if (textBoxSaveImageMask.Text.Trim().IndexOf("\\") > -1 && !CreateDestinationFolders())
                return;

            // Ulozeni MSL. Vyber jmena souboru.
            using (SaveFileDialog saveDlg = new SaveFileDialog())
            {
                saveDlg.Filter = "Magick Scripting Language (*.msl)|*.msl";
                saveDlg.RestoreDirectory = true;

                if (saveDlg.ShowDialog() == DialogResult.OK)
                {
                    // Vytvoreni XML stromu, ktery bude ulozen do souboru.
                    XDocument xdoc = GenerateImagesTree(destinationGeometry, selectedFilesCount);
                    if (xdoc != null)
                    {
                        // Vytvoreni souboru a ulozeni objektu XDocument.
                        XmlWriterSettings xmlSettings = new XmlWriterSettings();
                        xmlSettings.Encoding = new System.Text.UTF8Encoding(false);
                        xmlSettings.Indent = true;

                        XmlWriter xmlWriter = null;
                        try
                        {
                            xmlWriter = XmlWriter.Create(saveDlg.FileName, xmlSettings);

                            xdoc.WriteTo(xmlWriter);
                        }
                        catch (Exception ex)
                        {
                            MessageBox.Show(ex.Message, Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                            return;
                        }
                        finally
                        {
                            if (xmlWriter != null)
                                xmlWriter.Close();
                        }

                        // Spusteni noveho procesu pro konverzi obrazku.
                        StartConjure(saveDlg.FileName);
                    }
                }
            }
        }

        private XDocument GenerateImagesTree(StringBuilder destinationGeometry, int selectedFilesCount)
        {
            // Nova xml struktura.
            XDocument xdoc = new XDocument(new XDeclaration("1.0", "utf-8", null));
            // Korenovy element.
            XElement rootElement = new XElement("image");
            xdoc.Add(rootElement);

            // Zpracovani vsech souboru oznacenych pro export.
            int currentFileIndex = 0;
            foreach (DataGridViewRow row in dataGridViewFiles.Rows)
            {
                try
                {
                    if (Convert.ToInt32(row.Cells["Export"].Value) == 1)
                    {
                        // Zdrojovy soubor + zjisteni rozmeru obrazku.
                        XElement imageElement = new XElement("image",
                            new XElement("read",
                                new XAttribute("filename", row.Cells["FileName"].Value.ToString().Replace("\\", "\\\\"))
                            ),
                            new XElement("get",
                                new XAttribute("width", "base-width"),
                                new XAttribute("height", "base-height")
                            )
                        );

                        // Vysledne rozmery.
                        if (destinationGeometry.Length > 0)
                        {
                            imageElement.Add(new XElement("resize",
                                new XAttribute("geometry", destinationGeometry.ToString())),
                                new XElement("get",
                                    new XAttribute("width", "dest-width"),
                                    new XAttribute("height", "dest-height")
                                )
                            );
                        }

                        // Ulozeni v RGB.
                        if (checkBoxForceRgb.Checked)
                        {
                            imageElement.Add(new XElement("set",
                                new XAttribute("colorspace", "rgb")));
                        }

                        // Vypsani jmena zdrojoveho souboru.
                        imageElement.Add(new XElement("print",
                            new XAttribute("output", (++currentFileIndex).ToString() + "/" + selectedFilesCount.ToString()
                                + " " + row.Cells["FileName"].Value.ToString().Replace("\\", "\\\\") + "\\n")));

                        // Vypsani rozmeru zdrojoveho a ciloveho obrazku.
                        if (destinationGeometry.Length > 0)
                        {
                            imageElement.Add(new XElement("print",
                                new XAttribute("output", " zmena velikosti z %[base-width]x%[base-height] na %[dest-width]x%[dest-height]\\n")));
                        }

                        // Urceni jmena ciloveho souboru.
                        string srcFileName = row.Cells["FileName"].Value.ToString();
                        string fileDirectory = Path.GetDirectoryName(srcFileName);
                        string dstFileName = textBoxSaveImageMask.Text.Trim();

                        // Nahrazeni jmena souboru.
                        dstFileName = dstFileName.Replace("[:NAME:]", Path.GetFileNameWithoutExtension(srcFileName));

                        // Nahrazeni pripony.
                        if (comboBoxTargetFormat.SelectedIndex == 0)
                        {
                            // Zachovani formatu obrazku.
                            dstFileName = dstFileName.Replace("[:EXT:]", Path.GetExtension(srcFileName));
                        }
                        else
                        {
                            // Vynuceni formatu vybraneho v comboboxu.
                            dstFileName = dstFileName.Replace("[:EXT:]", "." + comboBoxTargetFormat.SelectedItem.ToString().ToLower());
                        }

                        dstFileName = Path.Combine(fileDirectory, dstFileName);

                        // Vypsani jmena ciloveho souboru.
                        imageElement.Add(new XElement("print",
                            new XAttribute("output", " ukladam jako " + dstFileName.Replace("\\", "\\\\") + "\\n")));

                        // Pro JPG ulozime i parametr udavajici kvalitu.
                        if (Path.GetExtension(dstFileName).ToLower() == ".jpg")
                        {
                            int quality;
                            if (!Int32.TryParse(maskedTextBoxJpgQuality.Text, out quality))
                            {
                                quality = 95;
                            }

                            if (quality < 1 || quality > 100)
                            {
                                quality = 95;
                                maskedTextBoxJpgQuality.Text = quality.ToString();
                            }

                            imageElement.Add(new XElement("write",
                                new XAttribute("filename", dstFileName.Replace("\\", "\\\\")),
                                new XAttribute("quality", quality)
                                ));
                        }
                        else
                        {
                            imageElement.Add(new XElement("write",
                                new XAttribute("filename", dstFileName.Replace("\\", "\\\\"))));
                        }

                        // Pridani zaznamu o obrazku ke korenovemu elemntu XML.
                        rootElement.Add(imageElement);
                    }
                }
                catch (InvalidCastException)
                {
                    return null;
                }
            }

            return xdoc;
        }

        private bool CreateDestinationFolders()
        {
            // Seznam obsahuje adresare, ve kterych se vyskytuji zdrojove obrazky.
            List<string> sourceDirectories = new List<string>();

            // Vyhledani vsech adresaru, ve kterych se obrazky vyskytuji.
            foreach (DataGridViewRow row in dataGridViewFiles.Rows)
            {
                try
                {
                    if (Convert.ToInt32(row.Cells["Export"].Value) == 1)
                    {
                        // Pridani adresaru do seznamu sourceDirectories.
                        string sourceDirectory = Path.GetDirectoryName(row.Cells["FileName"].Value.ToString());
                        if (!sourceDirectories.Contains(sourceDirectory))
                            sourceDirectories.Add(sourceDirectory);
                    }
                }
                catch (InvalidCastException)
                {
                    return false;
                }
            }

            // Vytvoreni vsech podadresaru.
            foreach (string sourceDirectory in sourceDirectories)
            {
                try
                {
                    string destinationDirectory = Path.Combine(sourceDirectory,
                        Path.GetDirectoryName(textBoxSaveImageMask.Text.Trim()));

                    Directory.CreateDirectory(destinationDirectory);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }
            }

            return true;
        }

        private void StartConjure(string mslFileName)
        {
            Process conjureProcess = new Process();
            // Spusteni prikazoveho radku.
            conjureProcess.StartInfo.FileName = "cmd.exe";
            // Nastaveni aktualniho adresare na cestu, kde je conjure.exe.
            conjureProcess.StartInfo.WorkingDirectory = Path.GetDirectoryName(textBoxConjurePath.Text.Trim());
            // Slozeni parametru pro cmd.exe
            // - vysledny proces je volan jako: cmd.exe /K conjure.exe "cesta k msl souboru"
            conjureProcess.StartInfo.Arguments = "/K " + Path.GetFileName(textBoxConjurePath.Text.Trim()) + " \"" + mslFileName + "\"";
            // Spusteni procesu.
            conjureProcess.Start();
        }

        private void dataGridViewFiles_DragEnter(object sender, DragEventArgs e)
        {
            //
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false) == true)
            {
                // Nacteni dat.
                string[] dragData = (string[])e.Data.GetData(DataFormats.FileDrop);
                // Dovolujeme pouze pretazeni 1 adresare.
                if (dragData.Length == 1 && Directory.Exists(dragData[0]))
                    e.Effect = DragDropEffects.Copy;
            }
        }

        private void dataGridViewFiles_DragDrop(object sender, DragEventArgs e)
        {
            // Ziskani dat a kontrola, jestli je vkladan pouze 1 adresar
            string[] dragData = (string[])e.Data.GetData(DataFormats.FileDrop);
            if (dragData.Length == 1 && Directory.Exists(dragData[0]))
            {
                // Nacteni souboru z vybraneho adresare.
                ImportDirectory(dragData[0]);
            }
        }
    }
}
