﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

// Source from http://thevalerios.net/matt/2008/05/a-queued-backgroundworker/
namespace Wiedemann
{
    public class DoWorkArgument<T>
    {
        public DoWorkArgument(T argument)
        {
            this.Argument = argument;
        }
        
        public T Argument { get; private set; }
    }

    public class WorkerResult<T>
    {
        public WorkerResult(T result, Exception error)
        {
            this.Result = result;
            this.Error = error;
        }

        public T Result { get; private set; }
        public Exception Error { get; private set; }
    }

    public class QueueItem<Tin>
    {
        public QueueItem(BackgroundWorker backgroundWorker, Tin argument)
        {
            this.BackgroundWorker = backgroundWorker;
            this.Argument = argument;
        }

        public Tin Argument { get; private set; }
        public BackgroundWorker BackgroundWorker { get; private set; }
    }

    public static class QueuedBackgroundWorker
    {
        public static void QueueWorkItem<Tin, Tout>(
            Queue<QueueItem<Tin>> queue,
            Tin inputArgument,
            Func<DoWorkArgument<Tin>, Tout> doWork,
            Action<WorkerResult<Tout>> workerCompleted)
        {
            if (queue == null) throw new ArgumentNullException("queue is null");

            BackgroundWorker bw = new BackgroundWorker();
            bw.WorkerReportsProgress = false;
            bw.WorkerSupportsCancellation = false;
            bw.DoWork += (sender, args) =>
            {
                if (doWork != null)
                {
                    args.Result = doWork(new DoWorkArgument<Tin>((Tin)args.Argument));
                }
            };
            bw.RunWorkerCompleted += (sender, args) =>
            {
                if (workerCompleted != null)
                {
                    workerCompleted(new WorkerResult<Tout>((Tout)args.Result, args.Error));
                }

                try
                {
                    queue.Dequeue();
                }
                catch (InvalidOperationException)
                {
                }

                if (queue.Count > 0)
                {
                    QueueItem<Tin> nextItem = queue.Peek();
                    nextItem.BackgroundWorker.RunWorkerAsync(nextItem.Argument);
                }
            };

            queue.Enqueue(new QueueItem<Tin>(bw, inputArgument));
            if (queue.Count == 1)
            {
                QueueItem<Tin> nextItem = queue.Peek();
                nextItem.BackgroundWorker.RunWorkerAsync(nextItem.Argument);
            }
        }

    }
}
